@@ -1,7 +1,8 @@
 import logging
 import RPi.GPIO as GPIO
+from time import time
 from typing import List
-from pi_awning_webthing.awning import Awning
+from pi_awning_webthing.awning import Awnings
 
 
 class Switch:
@@ -10,11 +11,11 @@ class Switch:
     MOVE_BACKWARD = (False, True)
     IDLE = (True, True)
 
-    def __init__(self, pin_forward: int, pin_backward: int, awnings: List[Awning]):
+    def __init__(self, pin_forward: int, pin_backward: int, awnings: Awnings):
         self.awnings = awnings
         self.pin_forward = pin_forward
         self.pin_backward = pin_backward
-        self.state = self.STOP
+        self.state = self.IDLE
         GPIO.setmode(GPIO.BCM)
         GPIO.setup(self.pin_forward, GPIO.IN, GPIO.PUD_DOWN)
         GPIO.add_event_detect(self.pin_forward, GPIO.BOTH)
@@ -24,32 +25,29 @@ class Switch:
         GPIO.add_event_callback(self.pin_backward, self.on_switch_updated)
         logging.info("Switch bound to pin_forward=" + str(self.pin_forward) + " and pin_backward=" + str(self.pin_backward))
 
-    def is_forward(self) -> bool:
-        return self.state[0]
-
-    def is_backward(self) -> bool:
-        return self.state[1]
-
     def on_switch_updated(self, pin: int):
         is_forward = GPIO.input(self.pin_forward) >= 1
         is_backward = GPIO.input(self.pin_backward) >= 1
         new_state = (is_forward, is_backward)
-        try:
-            if self.state != new_state:
-                logging.info("switch new state     " + str(new_state) + " (old " + str(self.state) + ")")
+        logging.info("\n\n\nnew state Forward=" + str(new_state[0]) + "; Backward=" + str(new_state[1]) + " is_moving=" + str(self.awnings.is_moving()))
+
+        if new_state != self.state:
+            try:
                 if new_state == self.MOVE_FORWARD:
-                    for anwing in self.awnings:
-                        anwing.set_target_position(100)
+                    if self.awnings.is_moving():
+                        logging.info("Forward and motion. stop")
+                        self.awnings.stop()
+                    else:
+                        logging.info("Forward set pos 0")
+                        self.awnings.set_position(100)
                 elif new_state == self.MOVE_BACKWARD:
-                    for anwing in self.awnings:
-                        anwing.set_target_position(0)
-                elif new_state == self.STOP:
-                    for anwing in self.awnings:
-                        current_pos = anwing.get_current_position()
-                        anwing.set_target_position(current_pos)
-                else:
-                    pass
-        except Exception as e:
-            logging.error(e)
-        finally:
-            self.state = new_state
+                    if self.awnings.is_moving():
+                        logging.info("Bckward and motion. stop")
+                        self.awnings.stop()
+                    else:
+                        logging.info("Bckward set pos 0")
+                        self.awnings.set_position(0)
+            except Exception as e:
+                logging.error(e)
+        else:
+            logging.info("double click")
